# Token生成示例代码（Golang）

示例代码：

``` go
package utils

import (
  "fmt"
  "github.com/golang-jwt/jwt/v5"
  "github.com/google/uuid"
  "github.com/stretchr/testify/assert"
  "testing"
  "time"
)

/*
-生成企业ServerToken

  uid: 实体（用户）唯一标识（可选）
  issuer: 企业英文缩写，例如：aihelp
  secretKey：在应用设置中生成的Auth Secret Key，例如：secret_59362f5481a04b2da21ea9c6a1750210
  expireSeconds: 过期时间（秒）
*/
func GenerateServerToken(subject string, issuer string, secretKey string, expireSeconds int) (string, error) {
  nowTime := time.Now()
  payload := jwt.RegisteredClaims{
    ID:      uuid.New().String(),
    Subject: subject,
    // 企业英文缩写
    Issuer: issuer,
    //过期时间戳（秒）
    ExpiresAt: jwt.NewNumericDate(nowTime.Add(time.Second * time.Duration(expireSeconds))),
    // 发布时间戳（秒）
    IssuedAt: jwt.NewNumericDate(nowTime),
    // 生效开始时间戳（秒）
    NotBefore: jwt.NewNumericDate(nowTime),
  }

  token := jwt.NewWithClaims(jwt.SigningMethodHS256, payload)
  signedJwt, err := token.SignedString([]byte(secretKey))
  if err != nil {
    return "", err
  }

  return signedJwt, nil
}

/*
 * 验证企业颁发的ServerToken
 */
func ValidateServerToken(key string, tokenString string) (*jwt.RegisteredClaims, error) {
  claims := &jwt.RegisteredClaims{}
  signingKey := []byte(key)
  token, err := jwt.ParseWithClaims(tokenString, claims, func(token *jwt.Token) (interface{}, error) {
    return signingKey, nil
  })

  if err != nil {
    return nil, err
  }

  if !token.Valid {
    return nil, fmt.Errorf("Invalid token")
  }
  return claims, nil
}

/*
*
测试生成Token
*/
func TestGenerateServerToken_ValidToken(t *testing.T) {
  subject := "user_110"
  issuer := "aihelp"
  secretKey := "secret_59362f5481a04b2da21ea9c6a1750210"
  expireSeconds := 86400

  token, err := GenerateServerToken(subject, issuer, secretKey, expireSeconds)
  assert.NoError(t, err)
  assert.NotEmpty(t, token)

  fmt.Println("生成的Token:", token)
}

/*
*
测试验证Token
*/
func TestValidServerToken(t *testing.T) {
  subject := "user_110"
  issuer := "aihelp"
  secretKey := "secret_59362f5481a04b2da21ea9c6a1750210"
  token := "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1aWQiOiJ1c2VyXzExMCIsImlzcyI6ImFpaGVscCIsInN1YiI6InVzZXJfMTEwIiwiZXhwIjoxNzMxNDc1NDU1LCJuYmYiOjE3MzE0NzE4NTUsImlhdCI6MTczMTQ3MTg1NSwianRpIjoiNzVhMmYxNGMtNTU1Yy00ZmIxLWE1ZTMtYjA3NzNiMDRhZmM3In0.J5zJ53qu8WCDsdsSH5OILcoPkWGdF_r367pF0CB0H-8"

  claims, err := ValidateServerToken(secretKey, token)

  assert.NoError(t, err, "无效的Token")
  assert.NotNil(t, claims, "无效的Token，没有解析出Claims")

  assert.Equal(t, issuer, claims.Issuer, "无效的Token，Issuer不匹配")
  assert.NotNil(t, claims.ExpiresAt, "无效的Token，ExpiresAt为空")
  assert.NotNil(t, claims.IssuedAt, "Claim中的IssuedAt为空，建议指定Token发布时间")
  assert.NotNil(t, claims.NotBefore, "Claim中的NotBefore为空，建议指定Token生效时间")

  assert.NotEmpty(t, claims.Subject, "Claim中的subject为空，建议指定实体（用户）唯一标识")
  if claims.Subject != "" {
    assert.Equal(t, subject, claims.Subject, "无效的Token，Uid不匹配")
  }
  //判断claims中的ID是否是UUID格式
  _, err = uuid.Parse(claims.ID)
  assert.NoError(t, err, "Claim中的ID不是UUID格式，建议使用UUID格式而不是固定的字符串，例如：uuid.New().String()")
}
```

